// File: builtinListener.cc


#include "builtinListener.h"
#include "spyAgent.h"

void myBuiltInPubListener::on_data_available(DDSDataReader* reader)
{
    printf("\nmyBuiltInPubListener on_data_available\n");
   
    DDSPublicationBuiltinTopicDataDataReader* builtin_reader = NULL;
    DDS_PublicationBuiltinTopicDataSeq        data_seq;
    DDS_SampleInfoSeq                         info_seq;
    DDS_ReturnCode_t                          retcode;
    DDS_ExceptionCode_t                       ex;
    int i;

    builtin_reader = DDSPublicationBuiltinTopicDataDataReader::narrow(reader);
    if (builtin_reader == NULL) {
        printf("Builtin Pub DataReader narrow error\n");
        return;
    }

    retcode = builtin_reader->take(
        data_seq, info_seq, DDS_LENGTH_UNLIMITED,
        DDS_ANY_SAMPLE_STATE, DDS_ANY_VIEW_STATE, DDS_ANY_INSTANCE_STATE);

    if (retcode == DDS_RETCODE_NO_DATA) {
        return;
    } else if (retcode != DDS_RETCODE_OK) {
        printf("take error %d\n", retcode);
        return;
    }

    for (i = 0; i < data_seq.length(); ++i) {
        if (info_seq[i].valid_data) {
            printf("Publication: topic '%s' of type '%s'\n", data_seq[i].topic_name,
                data_seq[i].type_name);
            data_seq[i].type_code->print_IDL(3, ex);                       
        }
    }

    retcode = builtin_reader->return_loan(data_seq, info_seq);
    if (retcode != DDS_RETCODE_OK) {
        printf("return loan error %d\n", retcode);
    }
   
}


void myBuiltInSubListener::on_data_available(DDSDataReader* reader)
{

    printf("\nmyBuiltInPubListener on_data_available\n");
   
    DDSSubscriptionBuiltinTopicDataDataReader* builtin_reader = NULL;
    DDS_SubscriptionBuiltinTopicDataSeq        data_seq;
    DDS_SampleInfoSeq                          info_seq;
    DDS_ReturnCode_t                           retcode;
    DDS_ExceptionCode_t                       ex;
    int i;

    builtin_reader = DDSSubscriptionBuiltinTopicDataDataReader::narrow(reader);
    if (builtin_reader == NULL) {
        printf("Builtin Sub DataReader narrow error\n");
        return;
    }

    retcode = builtin_reader->take(
        data_seq, info_seq, DDS_LENGTH_UNLIMITED,
        DDS_ANY_SAMPLE_STATE, DDS_ANY_VIEW_STATE, DDS_ANY_INSTANCE_STATE);

    if (retcode == DDS_RETCODE_NO_DATA) {
        return;
    } else if (retcode != DDS_RETCODE_OK) {
        printf("take error %d\n", retcode);
        return;
    }

    for (i = 0; i < data_seq.length(); ++i) {
        if (info_seq[i].valid_data) {
            printf("Subscription: topic '%s' of type '%s'\n", data_seq[i].topic_name,
                data_seq[i].type_name);
            data_seq[i].type_code->print_IDL(3, ex);
           
        }
    }

    retcode = builtin_reader->return_loan(data_seq, info_seq);
    if (retcode != DDS_RETCODE_OK) {
        printf("return loan error %d\n", retcode);
    }
   
};
