// File :  spyAgent.cc


#include "ndds/ndds_cpp.h"
#include "spyAgent.h"
#include "builtinListener.h"


SpyAgent::SpyAgent()
{
    // Constructor - initialization
}

int SpyAgent::startSpying(const int ddsDomainId)
{

    DDS_ReturnCode_t            retcode;
    DDSSubscriber               *builtin_subscriber = NULL;
    DDSDataReader               *builtin_pub_reader = NULL;
    DDSDataReader               *builtin_sub_reader = NULL;
    DDS_DomainParticipantQos    participant_qos;
    DDS_DomainParticipantFactoryQos factory_qos;
    struct NDDS_Transport_UDPv4_Property_t property =
                                NDDS_TRANSPORT_UDPV4_PROPERTY_DEFAULT;

    m_builtin_pub_listener  = new myBuiltInPubListener();
    m_builtin_sub_listener  = new myBuiltInSubListener();
      
   // To conserve memory, builtin Subscribers and DataReaders are created
   // only if and when you look them up. Therefore, to avoid missing 
   // builtin data, you should look up the builtin readers before the
   // DomainParticipant is enabled.
   
   // Modify the Factory's QoS so that DomainParticipants are not automatically enabled when created
    DDSTheParticipantFactory->get_qos(factory_qos);
    factory_qos.entity_factory.autoenable_created_entities = DDS_BOOLEAN_FALSE;
    DDSTheParticipantFactory->set_qos(factory_qos);
   
    // Modify participant_qos settings here, if necessary
    retcode = DDSTheParticipantFactory->get_default_participant_qos(participant_qos);
    if (retcode != DDS_RETCODE_OK) {
        printf("get default participant qos error %d\n", retcode);
        return -1;
    }

    // Increase type code serialized length from 256 (default) to 64K (maximum).
    // This ensures we can receive more involved type codes
    participant_qos.resource_limits.type_code_max_serialized_length = 0xffff;
   
    // Increase receiver buffer size. The UDPv4 transport resource settings
    // will be increased after m_participant is created.
    participant_qos.receiver_pool.buffer_size = 65535;
   
   // Create the DomainParticipant (which will start out disabled)
    m_participant = DDSTheParticipantFactory->create_participant(
            ddsDomainId, participant_qos,
            NULL /* listener */, DDS_STATUS_MASK_NONE);
    if (m_participant == NULL) {
        printf("create_participant error %d\n", retcode);
        //TODO : clean up?
        return -1;
    }

    // Change the default UPDv4 socket layer settings.
    // The default maximum data size is 8K. This is determined by the 9K maximum
    // socket buffer size and message_size_max, as defined in the UDPv4 builtin
    // transport.
    // To change the properties of the UDPv4 builtin transport, refer to the
    // online documentation (Programming How-To's ->Transport Use Cases).
    //      1. Turn off auto-enable of newly created participants (see above)
    //      2. Get the UDPv4 property
    //      3. Modify the property
    //      4. Set the property
    //      5. Enable the participant
    retcode = NDDSTransportSupport::get_builtin_transport_property(
                                    m_participant,
                                    DDS_TRANSPORTBUILTIN_UDPv4,
                                    (struct NDDS_Transport_Property_t&)property);
    if (retcode != DDS_RETCODE_OK) {
        printf("error getting builtin transport property: %d\n", retcode);
    }
   
    /* Increase the UDPv4 maximum message size to 64K (large messages) */
    property.parent.message_size_max = 65535;
    property.recv_socket_buffer_size = 65535;
    property.send_socket_buffer_size = 65535;
   
    retcode = NDDSTransportSupport::set_builtin_transport_property(
                                    m_participant,
                                    DDS_TRANSPORTBUILTIN_UDPv4,
                                    (struct NDDS_Transport_Property_t&)property);
    if (retcode != DDS_RETCODE_OK) {
        printf("error seting builtin transport property: %d\n", retcode);
    }   
   
   
    builtin_subscriber = m_participant->get_builtin_subscriber();
    if ( builtin_subscriber == NULL) {
        printf("error getting builtin_subscriber\n");
        return -1;
    }
   
    // builtin_reader for new publications: topic DDS_PUBLICATION_TOPIC_NAME
    m_builtin_pub_listener->agent = this;
    builtin_pub_reader = builtin_subscriber->lookup_datareader(DDS_PUBLICATION_TOPIC_NAME);
    if (builtin_pub_reader == NULL) {
        printf("error getting builtin_pub_reader\n");
        return -1;
    }
    retcode = builtin_pub_reader->set_listener(m_builtin_pub_listener, DDS_DATA_AVAILABLE_STATUS);
    if (retcode != DDS_RETCODE_OK) {
        printf("set_listener builtin_publication error\n");
        return -1;
    }
   
    // builtin_reader for new publications: topic DDS_SUBSCRIPTION_TOPIC_NAME
    m_builtin_sub_listener->agent = this;
    builtin_sub_reader = builtin_subscriber->lookup_datareader(DDS_SUBSCRIPTION_TOPIC_NAME);
    if (builtin_sub_reader == NULL) {
        printf("error getting builtin_sub_reader\n");
        return -1;
    }
    retcode = builtin_sub_reader->set_listener(m_builtin_sub_listener, DDS_DATA_AVAILABLE_STATUS);
    if (retcode != DDS_RETCODE_OK) {
        printf("set_listener builtin_sublication error %d\n", retcode);
        return -1;
    }
   
    // Enable the participant
    retcode = m_participant->enable();
   if (retcode != DDS_RETCODE_OK) {
        printf("participant enable error %d \n", retcode);
        return -1;
    }
       
}

int SpyAgent::stopSpying()
{
    // Clean up
}
